!-----------------------------------------------------------------------------------------------!
!   SEILER TIDE MODEL                                                                           !
!                                                                                               !
! orginal version from Ulrike Seiler, ulrike.seiler@zmaw.de                                     !
!                                                                                               !
! (c) Anna Fuchs 2012  , anna.fuchsmakaeva@googlemail.com                                       !
! (c) Petra Nerge 2012 , petra.nerge@informatik.uni-hamburg.de                                  !
!-----------------------------------------------------------------------------------------------!

PROGRAM globe

   USE mod_kind
   USE mod_declarate

   IMPLICIT NONE



   ometid   = degpeh * radian/3600._wp
   tidper   = zweipi/ometid
   delt     = tidper/itsper

! sine and cosine at latitudes
!   phi = 0.5_wp * dydeg * radian
!   DO j = lateu+1, lamv
!      cosu (j)        =  cos(phi)
!      sinu (j)        =  sin(phi)
!      cosu (lamu+2-j) =  cosu(j)
!      sinu (lamu+2-j) = -sinu(j)
!      cosur(j)        =  1./cosu(j)
!      cosur(lamu+2-j) =  cosur(j)
!      phi             =  phi+radian*dydeg
!   END DO
!
!   cosv (latev) = 1._wp
!   cosvr(latev) = 1._wp
!   sinv (latev) = 0._wp
!   phi = dydeg * radian
!   DO j = latev+1, lamv
!      cosv (j)           = cos(phi)
!      sinv (j)           = sin(phi)
!      cosv (lamv+2-j)    = cosv(j)
!      sinv (lamv+2-j)    = -sinv(j)
!      cosvr(j)           = 1._wp/cosvr(j)
!      cosvr(lamv+2-j)    = cosvr(j)
!   phi                = phi + radian * dydeg
!   END DO

! weighting coefficients for interpolation of quantities
   DO j=1, lafi
      gun(j) = 0._wp
      gus(j) = 0._wp
      gvn(j) = 0._wp
      gvs(j) = 0._wp
   END DO

   DO j = lafi, lamu
      gus(j) = cosv(j) * cosur(j)
      gun(j) = cosv(j+1) * cosur(j)
   END DO

   gus(lafi) = 0._wp
   gun(lamv) = 0._wp
   gus(lamv) = 0._wp
   DO j = lafip, lamv
      gvs(j) = cosu(j-1) * cosvr(j)
      gvn(j) = cosu(j) * cosvr(j)
   END DO

!coriolos-operator

   DO j = lafi, lamu
      fxc = omecor * sinu(j)
      fxcs(j) = sin(fxc * delt) / delt
   END DO

   DO j = lafip, lamv
      fyc = omecor * sinv(j)
      fycs(j) = sin(fyc * delt)/delt
   END DO

!sine and cosine at longitudes
!
!    DO i=1, lonv-1  !do 45
!       glon          = REAL(i) * dxdeg * radian
!       cosul(i)      = cos(glon)
!       sinul(i)      = sin(glon)
!       cosul(lonh-i) = -cosul(i)
!       sinul(lonh-i) = sinul(i)
!    END DO
!
!    cosul(lonv) = 0._wp
!    sinul(lonv) = 1._wp
!    cosul(lonh) = -1._wp
!    sinul(lonh) = 0._wp
!    DO i = lonh +1, longs -1
!       cosul(i) = cosul(longs-i)
!       sinul(i) = -sinul(longs-i)
!    END DO
!
!    cosul(longs) = 1._wp
!    sinul(longs) = 0._wp
!    DO i = 1, longs - 20  !do 55
!       ish = i + 20
!       acsul(i) = cosul(ish)
!       asnul(i) = sinul(ish)
!    END DO
!
!    DO i = longs -19, longs
!       ish = i - longs +20
!       acsul(i) = cosul(ish)
!       asnul(i) = sinul(ish)
!    END DO
!
!    DO i = 1, longs - 10
!       cosul(i)     = acsul(2 * i + 20)
!       sinul(i)     = asnul(2 * i + 20)
!       cosul(i + lonh) = cosul(i)
!       sinul(i + lonh) = sinul(i)
!    END DO
!
!    DO i = lonh - 9, lonh    !do 66
!       cosul(i) = acsul(2 * i + 20 - longs)
!       sinul(i) = asnul(2 * i + 20 - longs)
!       cosul(i + lonh) = cosul(i)
!       sinul(i + lonh) = sinul(i)
!    END DO
!
!    DO i = 1, lonv     !do 70
!       glon = (REAL (i) - 0.5_wp) * dxdeg * radian
!       cosvl(i) = cos(glon)
!       sinvl(i) = sin(glon)
!       cosvl(lonh + 1 - i) = -cosvl(i)
!       sinvl(lonh + 1 - i) = sinvl(i)
!    END DO
!
!    DO i = lonh + 1, longs
!       cosvl(i) = cosvl(longs + 1 -i)
!       sinvl(i) = -sinvl(longs +1 - i)
!    END DO
!
!    DO i = 1, longs - 20     !do 80
!       ish = i + 20
!       acsvl(i) = cosvl(ish)
!       asnvl(i) = sinvl(ish)
!    END DO
!
!    DO i = longs - 19, longs
!       ish = i - longs + 20
!       acsvl(i) = cosvl(ish)
!       asnvl(i) = sinvl(ish)
!    END DO
!
!    DO i = 1, lonh - 10
!       cosvl(i) = acsul(2 * i + 19)
!       sinvl(i) = asnul(2 * i + 19)
!       cosvl(i + lonh) = cosvl(i)
!       sinvl(i + lonh) = sinvl(i)
!    END DO
!
!    DO i = lonh - 9, lonh
!       cosvl(i) = acsul(2 * i + 19 - longs)
!       sinvl(i) = asnul(2 * i + 19 - longs)
!       cosvl(i+lonh) = cosvl(i)
!       sinvl(i+lonh) = sinvl(i)
!    END DO
!
!    DO i = 1, longs     !do 95
!       acc(i) = acsvl(i)* acsvl(i)
!       ass(i) = asnvl(i)* asnvl(i)
!       acs(i) = acsvl(i)* asnvl(i)
!    END DO

! complete reading of topography

    !rsta = unit(30)
    !IF (rsta.gt.0.) then
    !   write(6,'(23h input error on unit 30)')
    !  stop
    !ENDIF
    !close(30)

!!!! buffer in etc
! complete reading of initial data: zeta

    !rsta = uint(31)
!if(rsta.gt.0) THEN
  !write(6, '(23h input error on unit 31)')
      !stop
!END IF
!rewind 31

! calculate total depth at the beginning of a run

    res(:,:) = dep(:,:) + zeta(:,:)

! complete reading of initial data: velocities & budgets
    !DO iun = 32, 34
   !rsta = uint(iun)
   !IF(rsta.gt.0) THEN
     !write(6,'(21h input error on unit ,i2)') iun
 !stop
   !END IF
   !rewind iun
!END DO

!-begin timestepping

    !write(6,1020)

!-periods with fast diagnostics
    DO npe = 1, nper(1)
       itpar = itsper/npar(1)
       IF (npaa(1).eq.1) THEN
          resmap = -1._wp
          chovop =  0._dp
          buenp (:) = 0._dp
          buamrp (:) = 0._dp
       END IF


       DO npa = npaa(1), npae(1)   !do 250
          ita = (npa - 1) * itpar
          ite = npa * itpar - 1
          DO itt = ita, ite
             simtim = itt * delt
             phaset = ometid * simtim
             !-call step

             !calculate bulk quantities
             res (:,:) = dep (:,:) + zetaa(:,:)
             !call diagono(.true.)

             !-update variables

             zeta (:,:) = zetaa (:,:)
             DO j = lafi, latu      !do 190
                u (:, j)     = ua (:, j)
                v (:, j + 1) = va (:, j +1)
             END DO
          END DO

          !BUFFERS (Zeile 356 ff in pptimo.f)
          !buffer out (31,1) (zeta(0,lafi ),zeta (longp,lamz ))
	      !buffer out (32,1) (u   (0,lafi ),u    (longp,lamu ))
	      !buffer out (33,1) (v   (0,lafip),v    (longp,lamv ))
	      !buffer out (34,1) (buen(1      ),baamp(nteam,icomp))

          DO iun = 31, 34
          !wsta = unit(iun)
          !IF (wsta.gt.0._wp) THEN
             !write (6, '(22h output error on unit, i2)') iun
            ! STOP
          !END IF
          !rewind iun
          END DO
       END DO
       !output temporal mean bulk quantities (ZEILE 372 ff)

       IF (itt.eq.itsper) THEN
          buenp(:)  = buenp (:) / dble(itsper)
          buamrp(:) = buamrp (:) / dble (itsper)
          chovop    = chovop / dble (itsper)
          !write(6,1030) buenp, buamrp, chovop, resmap
       ELSE
          chovoi = dble(itpar * npae(1))
          buenp(:)  = buenp(:) / chovoi
          buamrp(:) = buamrp(:) / chovoi
          chovop    = chovop / chovoi
          !write(6,1040) npae(1), npar(1), buenp, buamrp, chovop, resmap
          buenp(:) = buenp(:) * chovoi
          buamrp(:) = buamrp(:) * chovoi
          chovop   = chovop * chovoi
       END IF

    END DO    !  300 continue

    !- periods with comprehensive diagnostics

    DO npe = 1, nper(2)
       itpar = itsper / npar(2)
       nprs  = npar(2) / 4
       IF (npaa(2).eq.1) THEN
          resmap = -1._wp
          chovop = 0._wp
          buenp(:)   = 0._wp !(DOUBLE)
          baenp(:)   = 0._wp
          baamp(:,:) = 0._wp
          buamrp(:)  = 0._wp
          bumorp(:)  = 0._wp
          bucomp(:)  = 0._wp
          buamap(:)   = 0._wp
          butoip(:)  = 0._wp
        END IF
        DO npa = npaa(2), npae(2)   !do 450
          iqu = npa / nprs
          npr = mod(npa, nprs)
          ita = (npa - 1) * itpar
          ite = npa * itpar - 1
          DO itt = ita, ite        !do 400
          simtim = itt * delt
          phaset = ometid * simtim
          !call step
          !-call diagnostics routine and output instantaneous bulk quantities
          IF (mod(itt, itbil).eq.0) THEN
             res(:,:) = dep(:,:) + zeta(:,:)
             !call bilanz
             res(:,:) = dep(:,:) + zetaa(:,:)
             !call diagno(.false.)

             !-write instantaneous bulks to binary file
             !-Zeile 441 ++ pptimo.f

          ELSE
             res(:,:) = dep(:,:) + zetaa(:,:)
          END IF
        !-update variables
       zeta(:,:) = zetaa(:,:)
       DO j = lafi, latu       ! 390
          u(:,j)     = ua(:,j)
          v(:,j + 1) = va(:,j + 1)
       END DO
    END DO

        ! save arrays at the end of each partition

        !buffer out (31,1) (zeta(0,lafi ),zeta (longp,lamz ))
        !buffer out (32,1) (u   (0,lafi ),u    (longp,lamu ))
        !buffer out (33,1) (v   (0,lafip),v    (longp,lamv ))
        !buffer out (34,1) (buen(1      ),baamp(nteam,icomp))
        !wsta = unit(31)
        !IF(wsta.gt.0.) THEN
        !write(6,'(24h output error on unit 31)')
          !STOP
        !END IF
        !rewind 31
        IF(pham.and.npr.eq.0) print*, 'test' !call result(iqu)
        DO iun = 32, 34
              !wsta = unit(iun)
              ! // HIER FEHLR EIN END IF ! SIEHE pptimo.f ZEILE 469 ff
              !IF (wsta.gt.0.) THEN
                  !write(6,'(22h output error on unit ,i2)') iun
                  !STOP
              !END IF
              !rewind iun
        END DO
    END DO

    !output of temporal mean bulk quantities

    IF (itt.eq.itsper) THEN
       chovoi = dble(itsper / itbil)
       buenp(:) = buenp / chovoi
       baenp(:) = baenp(:) / chovoi
       baamp(:,:) = baamp(:,:) / chovoi
       buamrp(:) = buamrp(:) / chovoi
       bumorp(:) = bumorp(:) / chovoi
       bucomp(:) = bucomp(:) / chovoi
       buamap(:) = buamap(:) / chovoi
       butoip(:) = butoip(:) / chovoi
       chovop = chovop / chovoi
       !write(61) buenp .. Zeile 506

    ELSE
       chovoi = dble(itpar * npae(2) / itbil)
       buenp(:)  = buenp(:) / chovoi
       buamrp(:) = buamrp(:) / chovoi
       chovop    = chovop / chovoi
       !write(6, 1040) npae(2), npar(2), buenp, buamrp, chovop, resmap
       buenp(:)  = buenp(:) * chovoi
       buamrp(:) = buamrp(:) * chovoi
       chovop    = chovop * chovoi
    END IF
    END DO


END PROGRAM


