!-----------------------------------------------------------------------------------------------!
! Module for nice to have math routines                                                         !
!                                                                                               ! 
! (c) Petra Nerge 2012, petra.nerge@informatik.uni-hamburg.de                                   !  
!-----------------------------------------------------------------------------------------------!
!                                                                                               ! 
!   Description                                                                                 ! 
!                                                                                               !
!   This module contains subroutines to easy often used mathematical applications.              ! 
!                                                                                               ! 
!   Interface contain                                                                           ! 
!   - nice_math_invert:                                                                         !
!     - get invers variable                                                                     !
!                                                                                               !   
!-----------------------------------------------------------------------------------------------!

MODULE mod_nice_math

    USE mod_kind

    IMPLICIT none
    !--- Unit number of write error messages
    INTEGER , PARAMETER :: nice_math_eunit = 0

    !--- Invert input variable
    INTERFACE nice_math_invert
            MODULE PROCEDURE nice_math_invert_float
            MODULE PROCEDURE nice_math_invert_double
    END INTERFACE nice_math_invert
    !--- Square of input variable
    INTERFACE nice_math_square
            MODULE PROCEDURE nice_math_square_float
            MODULE PROCEDURE nice_math_square_double
    END INTERFACE nice_math_square
    !--- Multiplication of input variables
    INTERFACE nice_math_mul
            MODULE PROCEDURE nice_math_mul_float
            MODULE PROCEDURE nice_math_mul_double
    END INTERFACE nice_math_mul


  !---------------------------------------------------------------------------------------------!

  CONTAINS

    !-------------------------------------------------------------------------------------------!

    ELEMENTAL FUNCTION nice_math_invert_float( var )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !      
    !     - inverse input variable of type float                                                !
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - variable of type float                                                              !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------! 

        !--- Function
        REAL (KIND=sp) :: nice_math_invert_float

        !--- Function Variables
        !--- Variable to invert
        REAL (KIND=sp)  , INTENT (IN) :: var 

        
        !---------------------------------------------------------------------------------------!
        !--- Invert variable var ---------------------------------------------------------------! 
        
        nice_math_invert_float = 1._sp / var
        
        !---------------------------------------------------------------------------------------!

    END FUNCTION nice_math_invert_float

    !-------------------------------------------------------------------------------------------!

    ELEMENTAL FUNCTION nice_math_invert_double( var )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !      
    !     - inverse input variable of type double                                               !
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - variable of type double                                                             !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------! 

        !--- Function
        REAL (KIND=dp) :: nice_math_invert_double

        !--- Function Variables
        !--- Variable to invert
        REAL (KIND=dp)  , INTENT (IN) :: var 

        
        !---------------------------------------------------------------------------------------!
        !--- Invert variable var ---------------------------------------------------------------! 
        
        nice_math_invert_double = 1._dp / var
        
        !---------------------------------------------------------------------------------------!

    END FUNCTION nice_math_invert_double

    !-------------------------------------------------------------------------------------------!

    ELEMENTAL FUNCTION nice_math_square_float( var )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !      
    !     - square input variable of type float                                                 !
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - variable of type float                                                              !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------! 

        !--- Function
        REAL (KIND=sp) :: nice_math_square_float

        !--- Function Variables
        !--- Variable to invert
        REAL (KIND=sp)  , INTENT (IN) :: var 

        
        !---------------------------------------------------------------------------------------!
        !--- Square of variable ----------------------------------------------------------------! 
        
        nice_math_square_float = var * var
        
        !---------------------------------------------------------------------------------------!

    END FUNCTION nice_math_square_float

    !-------------------------------------------------------------------------------------------!

    ELEMENTAL FUNCTION nice_math_square_double( var )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !      
    !     - square input variable of type double                                                !
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - variable of type double                                                             !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------! 

        !--- Function
        REAL (KIND=dp) :: nice_math_square_double

        !--- Function Variables
        !--- Variable to invert
        REAL (KIND=dp)  , INTENT (IN) :: var 

        
        !---------------------------------------------------------------------------------------!
        !--- Square of variable  ---------------------------------------------------------------! 
        
        nice_math_square_double = var * var
        
        !---------------------------------------------------------------------------------------!

    END FUNCTION nice_math_square_double

    !-------------------------------------------------------------------------------------------!

    ELEMENTAL FUNCTION nice_math_mul_float( var1 , var2 )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !      
    !     - multiplication of input variables of type float                                     !
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - variables of type float                                                             !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------! 

        !--- Function
        REAL (KIND=sp) :: nice_math_mul_float

        !--- Function Variables
        !--- Variable to multilpy
        REAL (KIND=sp)  , INTENT (IN) :: var1 , var2 

        
        !---------------------------------------------------------------------------------------!
        !--- Multiplication of variables -------------------------------------------------------! 
        
        nice_math_mul_float = var1 * var2
        
        !---------------------------------------------------------------------------------------!

    END FUNCTION nice_math_mul_float

    !-------------------------------------------------------------------------------------------!

    ELEMENTAL FUNCTION nice_math_mul_double( var1, var2 )

    !-------------------------------------------------------------------------------------------!    
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - Return value                                                                          !      
    !     - multiplication of input variables of type double                                    !
    !                                                                                           !
    !   - Input                                                                                 !  
    !     - variables of type double                                                            !  
    !                                                                                           !
    !-------------------------------------------------------------------------------------------! 

        !--- Function
        REAL (KIND=dp) :: nice_math_mul_double

        !--- Function Variables
        !--- Variable to invert
        REAL (KIND=dp)  , INTENT (IN) :: var1 , var2 

        
        !---------------------------------------------------------------------------------------!
        !--- Multiplication of variables -------------------------------------------------------! 
        
        nice_math_mul_double = var1 * var2
        
        !---------------------------------------------------------------------------------------!

    END FUNCTION nice_math_mul_double

    !-------------------------------------------------------------------------------------------!

END MODULE mod_nice_math

 
