!------------------------------------------------------------------------------------------------!
! Module of time stepping                                                                        !
!                                                                                                !
! orginal version from Ulrike Seiler , ulrike.seiler@zmaw.de                                     !
!                                                                                                !
! (c) Petra Nerge 2012 , petra.nerge@informatik.uni-hamburg.de                                   !
!------------------------------------------------------------------------------------------------!
!                                                                                                !
! Description                                                                                    !
!                                                                                                !
!   Module of time stepping                                                                      !


MODULE mod_time_stepping

    USE mod_kind
    USE mod_tide_force , ONLY : tide_period

    IMPLICIT NONE

    !--- GLOBAL VARIABLES CONTAIN ---------------------------------------------------------------!
    !
    !--- Time stepping
    !--- Number of time steps per tidal period
    INTEGER                                        :: timesteps_tide_period    !--- org: itsper
    !--- Number of tidal periods to be calculated
    INTEGER                                        :: number_tide_periods
    !--- Time step in secs
    REAL (KIND=wp)                                 :: delta_time               !--- org: delt
    !--- Simulation time
    REAL (KIND=wp)                                 :: simulation_time          !--- org: simtim
    !--- Phase lag of the tide
    REAL (KIND=wp)                                 :: phase_lag_tide           !--- org: phaset
    !--- Cosine of phase lag of the tide
    REAL (KIND=wp)                                 :: cos_phase_lag_tide       !--- org: cosomt
    !--- Sine of phase lag of the tide
    REAL (KIND=wp)                                 :: sin_phase_lag_tide       !--- org: sinomt
    !--------------------------------------------------------------------------------------------!


    !--- INTERFACE CONTAIN ----------------------------------------------------------------------!
    !
    !--- Initialize of the time stepping variables
    INTERFACE time_stepping_initialize
            module procedure time_stepping_initialize
    END INTERFACE time_stepping_initialize


  !---------------------------------------------------------------------------------------------!

  CONTAINS

    !-------------------------------------------------------------------------------------------!

    SUBROUTINE time_stepping_initialize( )

    !-------------------------------------------------------------------------------------------!
    !                                                                                           !
    !   Description                                                                             !
    !                                                                                           !
    !   - initializes the time stepping variables                                               !
    !                                                                                           !
    !-------------------------------------------------------------------------------------------!

        IMPLICIT none


        !---------------------------------------------------------------------------------------!
        !--- Initialize time stepping ----------------------------------------------------------!

        !--- Number of time steps per tidal period
        timesteps_tide_period = 100
        !--- Number of tidal periods to be calculated
        number_tide_periods   = 20
        !--- Time step
        delta_time = tide_period / REAL( timesteps_tide_period , wp )
        !--- Simulation time
        simulation_time = 0._wp
        !--- Phase lag of the tide
        phase_lag_tide = 0._wp 
        !--- Cosine of phase lag of the tide
        cos_phase_lag_tide = 0._wp 
        !--- Sine of phase lag of the tide
        sin_phase_lag_tide = 0._wp 

        !---------------------------------------------------------------------------------------!

    END SUBROUTINE time_stepping_initialize

    !-------------------------------------------------------------------------------------------!

END MODULE mod_time_stepping

